package com.ichangzuo.system.service;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ichangzuo.common.base.BasePageQuery;
import com.ichangzuo.common.enums.ContactType;
import com.ichangzuo.common.result.Result;
import com.ichangzuo.system.model.dto.UserAuthInfo;
import com.ichangzuo.system.model.dto.UserExportDTO;
import com.ichangzuo.system.model.entity.*;
import com.ichangzuo.system.model.form.*;
import com.ichangzuo.system.model.query.DrawQuery;
import com.ichangzuo.system.model.query.UserPageQuery;
import com.ichangzuo.system.model.vo.TakeOrderPageVO;
import com.ichangzuo.system.model.vo.UserInfoVO;
import com.ichangzuo.system.model.vo.UserPageVO;
import com.ichangzuo.system.model.vo.UserProfileVO;

import java.util.List;

/**
 * 用户业务接口
 *
 * @author haoxr
 * @since 2022/1/14
 */
public interface UserService extends IService<User> {

    /**
     * 用户分页列表
     *
     * @return
     */
    IPage<UserPageVO> listPagedUsers(UserPageQuery queryParams);


    /**
     * 获取用户表单数据
     *
     * @param userId
     * @return
     */
    UserForm getUserFormData(Long userId);


    /**
     * 新增用户
     *
     * @param userForm 用户表单对象
     * @return
     */
    boolean saveUser(UserForm userForm);

    /**
     * 修改用户
     *
     * @param userId   用户ID
     * @param userForm 用户表单对象
     * @return
     */
    boolean updateUser(Long userId, UserForm userForm);


    /**
     * 删除用户
     *
     * @param idsStr 用户ID，多个以英文逗号(,)分割
     * @return
     */
    boolean deleteUsers(String idsStr);

    /**
     * 锁定/恢复用户
     *
     * @param idsStr 用户ID，多个以英文逗号(,)分割
     * @return
     */
    boolean setUsersState(String idsStr, int state);

    /**
     * 根据用户名获取认证信息
     *
     * @param username 用户名
     * @return {@link UserAuthInfo}
     */

    UserAuthInfo getUserAuthInfo(String username);


    /**
     * 获取导出用户列表
     *
     * @param queryParams 查询参数
     * @return
     */
    List<UserExportDTO> listExportUsers(UserPageQuery queryParams);


    /**
     * 获取登录用户信息
     *
     * @return
     */
    UserInfoVO getCurrentUserInfo();

    /**
     * 获取个人中心用户信息
     *
     * @return
     */
    UserProfileVO getUserProfile(Long userId);

    /**
     * 修改个人中心用户信息
     *
     * @param formData 表单数据
     * @return
     */
    boolean updateUserProfile(UserProfileForm formData);

    /**
     * 修改用户密码
     *
     * @param userId 用户ID
     * @param data   修改密码表单数据
     * @return
     */
    boolean changePassword(Long userId, PasswordChangeForm data);

    /**
     * 重置用户密码
     *
     * @param userId   用户ID
     * @param password 重置后的密码
     * @return
     */
    boolean resetPassword(Long userId, String password);

    Result<?> sendResetPasswordCode(String username, String contact);

    /**
     * 发送验证码
     *
     * @param contact 联系方式
     * @param title 发送标题
     * @return
     */
    boolean sendVerificationCode(String contact, String title);
    boolean sendVerificationCode(String contact, String title, ContactType contactType);
    /**
     * 修改当前用户手机号
     *
     * @param data 表单数据
     * @return
     */
    boolean bindMobile(MobileBindingForm data);

    /**
     * 修改当前用户邮箱
     *
     * @param data 表单数据
     * @return
     */
    boolean bindEmail(EmailChangeForm data);

    /**
     * 重置密码验证
     *
     * @param contact
     * @param code
     * @return
     */
    int checkVerificationCode(String contact, String code, String title);

    void afterUserLogon();
    Account getCurrentAccount();
    Account getAccountById(Long userId);
    RealNameInfo getCurrentRealNameInfo();
    boolean submitRealName(RealNameInfo info);
    boolean verifyRealName(Long userId, String rejectReason);
    int beingVIP(Integer vipType);
    Result<?> buyNotes(String payType, String amount);
    Result<?>  drawNotes(DrawQuery draw);
    IPage<TakeOrderPageVO> listPageDraw(BasePageQuery queryParams);
    int verifyDraw(Integer orderId, String rejectReason);
    boolean existUsername(String username);
    boolean register(String username, String password);
    Integer deleteUser(String username, String password);
    User verifyUser(String username, String password);
    boolean addNotes(Long userId, Integer amount, Integer time);
    Integer checkOrder(String order);
    void setNewAccount();
}
